import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'datamodel.dart';
import 'sensorconnector.dart';


class Configuration extends StatefulWidget {
  const Configuration({Key? key}) : super(key: key);

  @override
  State<Configuration> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<Configuration> {
  late Future<Collection> futureCollection;
  late Future<List<Collection>> futureCollections;
  late Future<List<EventType>> futureEventTypes;
  late Future<List<Device>> futureDevices;
  late Future<String> futureAuthToken;

  SensorConnector connector = SensorConnector();
  final ConfigurationStoreInstance configuration = ConfigurationStoreInstance();

  @override
  void initState() {
    super.initState();

    //Start fetching all possible sensor data as soon as possible:
    futureCollections = connector.fetchCollections();
    futureEventTypes = connector.fetchEventTypes();
    futureDevices = connector.fetchCollectionDevices(configuration.currentCollection.id);
    futureAuthToken = connector.getAuthToken(configuration.loginInformation.mail, configuration.loginInformation.password);
  }
  late Future<String> eventTypes;

  @override
  Widget build(BuildContext context) {

    final ConfigurationStoreInstance configuration = ConfigurationStoreInstance();
    final EventStoreInstance events = EventStoreInstance();

    return Scaffold(
      appBar: AppBar(
        title: const Text('Configuration'),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            const Text(
                'You must be online to do something here!',
                style: TextStyle(fontSize: 14)
            ),
            const SizedBox(height: 50),
            TextFormField(
              keyboardType: TextInputType.emailAddress,
              autofocus: false,
              initialValue: configuration.loginInformation.mail,
              decoration: const InputDecoration(
                labelText: 'Sensor E-Mail',
                hintText: 'User for writing events',
              ),
              onChanged: (value) {
                configuration.loginInformation.mail = value;
              },
              onFieldSubmitted: (value) {
                futureAuthToken = connector.getAuthToken(configuration.loginInformation.mail, configuration.loginInformation.password);
              },
            ),
            TextFormField(
              autofocus: false,
              initialValue: configuration.loginInformation.password,
              decoration: const InputDecoration(
                labelText: 'Sensor Password',
                hintText: 'Password for writing events',
              ),
              onChanged: (value){
                configuration.loginInformation.password = value;
              },
              onFieldSubmitted: (value) {
                futureAuthToken = connector.getAuthToken(configuration.loginInformation.mail, configuration.loginInformation.password);
              },
            ),
            const SizedBox(height: 50),
            FutureBuilder<List<Collection>>(
                future: futureCollections,
                builder: (context, snapshot){
                  if (snapshot.hasData)
                  {
                    configuration.collections = [];
                    snapshot.data?.forEach((element) {
                      configuration.collections.add(element);
                    });

                    /*Initialize active collection with first received
                    collection if not initialized yet*/
                    if(configuration.currentCollection.id == -1){
                      configuration.currentCollection = configuration.collections[0];
                    }

                    return DropdownButtonFormField(
                        value: configuration.currentCollection.collectionName,
                        decoration: const InputDecoration(
                          labelText: 'Chose Collection',
                        ),
                        items:
                        configuration.collections.map((Collection collection) {
                          return DropdownMenuItem(
                            value: collection.collectionName,
                            child: Text(collection.collectionName),
                          );
                        }).toList(),
                        onChanged: (value) {
                          configuration.currentCollection = configuration.getCollectionFromName(value.toString());
                          //Fetch new selected collection devices
                          futureDevices = connector.fetchCollectionDevices(configuration.currentCollection.id);
                        }
                    );
                  }
                  else{
                    return const CircularProgressIndicator();
                  }
                }
            ),
          ],
        ),
      ),
      bottomNavigationBar: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          FloatingActionButton.extended(
            heroTag: null,
            tooltip: 'Reset all data',
            icon: const Icon(Icons.cancel),
            label: const Text('Reset all'),
            onPressed: () {
              //events.reset();  //TODO: remove before release!
              configuration.reset();  //TODO: remove before release this is not a real use case!
            },
          ),
          FloatingActionButton.extended(

            heroTag: null,
            tooltip: 'Select Collection and download corresponding devices',
            icon: const Icon(Icons.save),
            label: const Text('Store'),
            onPressed: () {
              futureAuthToken.whenComplete((){
                futureDevices.whenComplete(() {
                  //Update configuration with newly fetched data
                  futureDevices.then((value) => configuration.devices = value);
                  futureEventTypes.then((value) => configuration.eventTypes = value);

                  //Update id and urn for the add event widget with some initial data
                  events.currentEvent.id = configuration.devices[0].id;
                  events.currentEvent.urn = configuration.devices[0].urn;
                  events.currentEvent.description = '';
                  events.currentEvent.label = '';
                  events.currentEvent.type = configuration.eventTypes[0].name;

                  var date = DateTime.now().toUtc();
                  var isodata = date.toIso8601String();
                  events.currentEvent.startDate = isodata;
                  events.currentEvent.endDate = isodata;
                  configuration.initialized = true;

                  HapticFeedback.vibrate();
                });
              });


              //login();
              //fetchEventTypes();
              //updateDevices(configuration.currentCollection.id);
              //HapticFeedback.vibrate();
            },
          ),
        ],
      ),
    );
  }
}
